# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator/create_python_api.py script.
"""Public API for tf.config namespace.
"""

from __future__ import print_function as _print_function

from tensorflow._api.v1.compat.v2.config import experimental
from tensorflow._api.v1.compat.v2.config import optimizer
from tensorflow._api.v1.compat.v2.config import threading
from tensorflow.python import connect_to_remote_host as experimental_connect_to_host
from tensorflow.python.eager.context import list_devices as experimental_list_devices
from tensorflow.python.eager.def_function import run_functions_eagerly as experimental_run_functions_eagerly
from tensorflow.python.framework.config import get_soft_device_placement
from tensorflow.python.framework.config import set_soft_device_placement

del _print_function
