# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator/create_python_api.py script.
"""Loss operations for use in neural networks.

Note: All the losses are added to the `GraphKeys.LOSSES` collection by default.

"""

from __future__ import print_function as _print_function

from tensorflow.python.ops.losses.losses import Reduction
from tensorflow.python.ops.losses.losses import absolute_difference
from tensorflow.python.ops.losses.losses import add_loss
from tensorflow.python.ops.losses.losses import compute_weighted_loss
from tensorflow.python.ops.losses.losses import cosine_distance
from tensorflow.python.ops.losses.losses import get_losses
from tensorflow.python.ops.losses.losses import get_regularization_loss
from tensorflow.python.ops.losses.losses import get_regularization_losses
from tensorflow.python.ops.losses.losses import get_total_loss
from tensorflow.python.ops.losses.losses import hinge_loss
from tensorflow.python.ops.losses.losses import huber_loss
from tensorflow.python.ops.losses.losses import log_loss
from tensorflow.python.ops.losses.losses import mean_pairwise_squared_error
from tensorflow.python.ops.losses.losses import mean_squared_error
from tensorflow.python.ops.losses.losses import sigmoid_cross_entropy
from tensorflow.python.ops.losses.losses import softmax_cross_entropy
from tensorflow.python.ops.losses.losses import sparse_softmax_cross_entropy

del _print_function

import sys as _sys
from tensorflow.python.util import deprecation_wrapper as _deprecation_wrapper

if not isinstance(_sys.modules[__name__], _deprecation_wrapper.DeprecationWrapper):
  _sys.modules[__name__] = _deprecation_wrapper.DeprecationWrapper(
      _sys.modules[__name__], "losses")
