#  tests for tensorflow-base-1.14.0-eigen_py37hf4a566f_0 (this is a generated file);
print('===== testing package: tensorflow-base-1.14.0-eigen_py37hf4a566f_0 =====');
print('running run_test.py');
#  --- run_test.py (begin) ---
import tensorflow as tf
hello = tf.constant('Hello, TensorFlow!')
sess = tf.Session()
out = sess.run(hello)
a = tf.constant(10)
b = tf.constant(32)
sess.run(a+b)
#  --- run_test.py (end) ---

print('===== tensorflow-base-1.14.0-eigen_py37hf4a566f_0 OK =====');
print("import: 'tensorflow'")
import tensorflow

